//
// Testcases for 'fl_filename_relative()'
//
#include <stdio.h>
#include <FL/filename.H>
#include <cstring>

int main(int/* argc*/, char **/*argv*/)
{
	struct base_from
	{
		const char *base;
		const char *from;
		const char *result; // expected result (or leave empty, if result not known)
	};
	struct base_from testsuite[] = {
		{
			// STR-3441
			"/test/path_suffix/folder",
			"/test/path/folder/file",
			"../../path/folder/file"
		},
		{
			// STR-3441 + '/' at end of folder => should be ignored
			"/test/path_suffix/folder/",
			"/test/path/folder/file",
			"../../path/folder/file"
		},
		{
			// handle multiple '/' in source dir (base) correctly
			"/test/path_suffix////folder",
			"/test/path/folder/file",
			"../../path/folder/file"
		},
		{
			// handle multiple '/' in dest. dir (from) correctly (keep them)
			"/test/path_suffix/folder",
			"/test/path/folder/////file",
			"../../path/folder/////file"
		},
		{
			// handle multiple '/' correctly (combined previous 2 tests)
			"/test/path_suffix///folder",
			"/test/path/folder/////file",
			"../../path/folder/////file"
		},
		{
			// STR-2384 (reverse of STR-3441)
			"/test/path/folder/folder1/file",
			"/test/path_suffix/folder/file",
			"../../../../path_suffix/folder/file"
		},
		{
			// destination path (from) is '/'
			"/test/path/folder",
			"/",
			"../../../"
		},
		{
			// source path (base) is '/'
			"/",
			"/test/path/folder/file",
			"test/path/folder/file"
		},
		{
			// STR-3441 variant
			"/test/path/folder1",
			"/test/path_suffix/folder/file",
			"../../path_suffix/folder/file"
		},
		{
			// another test for STR-2384
			"/anydir/test/subdir",
			"/anydir/test1/subdir/file",
			"../../test1/subdir/file"
		},
		{
			// another test for STR-3441
			"/anydir/test1/subdir",
			"/anydir/test/subdir/file",
			"../../test/subdir/file"
		},
		{
			// same path but appended '/'
			"/test/path",
			"/test/path/",
			""		// NOTE: this sould IMHO better return "./"
		},
		{
			// exactly same path
			"/test/path",
			"/test/path",
			"."	// NOTE: this sould IMHO better return "./"
		},
		{
			// same path with file
			"/test/path/folder",
			"/test/path/folder/file",
			"file"
		}
	};

	int failed = 0;
	int not_checked = 0;
	for (int i = 0; i < sizeof(testsuite) / sizeof(testsuite[0]); i++)
	{
		char to[FL_PATH_MAX] = "";
		const char *base = testsuite[i].base;
		const char *from = testsuite[i].from;
		const char *result = testsuite[i].result;

		printf("TEST %d:\n'%s' => '%s'\n", i+1, base, from);

		int to_valid = fl_filename_relative(to, sizeof(to), from, base);
		if (!to_valid)
		{
			printf("***INVALID TEST\n");
			return 0;
		}
		printf("'%s'\n", to);
		if (!result)
			not_checked++;
		else if (!strcmp(to, result) == 0)
		{
			printf("***FAILED***: expected '%s'\n", result);
			failed++;
		}
		else
		{
			printf("PASSED\n");
		}
		printf( "-------------------------------------------------\n" );
	}
	if (failed == 0 && not_checked == 0)
		printf("*** SUCCESS! ***\n");
	if (failed)
		printf("*** %d FAILED TESTS! ****\n", failed);
	if (not_checked)
		printf("*** %d TESTS NEED TO BE VERIFIED ***\n", not_checked);

	return 0;
}
